import threading
import numpy as np
from enum import Enum


class GNautilus:

    class Sensitivities(Enum):
        SENS_2250000 = 2250000,
        SENS_1125000 = 1125000,
        SENS_750000 = 750000,
        SENS_562500 = 562500,
        SENS_375000 = 375000,
        SENS_187500 = 187500

    _serial: str
    _sampling_rate: float
    _channel_count: int
    _device: None
    _data_callback: callable
    _running: bool

    @staticmethod
    def register(key: str):
        from .lib.gtec_gds_wrapper import GDS
        GDS.register(key=key)

    def __init__(self,
                 serial: str = None,
                 sampling_rate: float = None,
                 channel_count: int = None,
                 sensitivity: float = None,
                 **kwargs):
        self._serial = serial
        self._sampling_rate = None
        self._channel_count = None
        self._sensitivity = None
        self._device = None
        self._data_callback = None
        self._running = False

        from .lib.gtec_gds_wrapper import GDS, ConnectedDevices
        GDS._check_key()

        if sampling_rate is None:
            raise ValueError("Sampling rate not set.")

        cd = ConnectedDevices()
        if len(cd) == 0:
            raise Exception("No GDS devices found.")
        self._device = GDS(gds_device=cd[0][0])
        if channel_count is None:
            channel_count = len(self._device.Channels)
        supported_sens = self._device.GetSupportedSensitivities()[0]
        if sensitivity is None:
            sensitivity = supported_sens[0]  # lowest sensitivity per default
        if sensitivity not in supported_sens:
            raise ValueError("Sensitivity not supported. Please use one "
                             "of {}.".format(supported_sens))
        for i in range(len(self._device.Channels)):
            self._device.Channels[i].Enabled = i < channel_count
            self._device.Channels[i].Sensitivity = sensitivity
        self._device.SamplingRate = sampling_rate

        self._device.SetConfiguration()
        self._sampling_rate = sampling_rate
        self._channel_count = channel_count
        self._sensitivity = sensitivity

    def __del__(self):
        if self._device is not None:
            del self._device
            self._device = None

    def start(self):
        if self._data_callback is None:
            raise ValueError("No data callback set.")
        self._running = True
        self._data_thread = threading.Thread(target=self._data_thread_fun)
        self._data_thread.daemon = True
        self._data_thread.start()

    def stop(self):
        self._running = False
        if self._data_thread:
            self._data_thread.join()

    def set_data_callback(self, callback):
        self._data_callback = callback

    def _data_thread_fun(self):
        self._device.GetData(scanCount=1, more=self._callback_wrapper)

    def get_impedance(self, first: bool = True):
        z = self._device.GetImpedanceEx(electrodeType=0,  # passive electrodes
                                        tmsBoxCorrection=False,
                                        firstMeasurementOfSession=first)
        return np.array(z[0][:self._channel_count])

    def _callback_wrapper(self, data: np.ndarray) -> bool:
        self._data_callback(data.copy())
        return self._running

    @property
    def get_serial_number(self):
        return self._serial

    @property
    def get_sampling_rate(self):
        return self._sampling_rate

    @property
    def get_channel_count(self):
        return self._channel_count
